function [maxlag_selected, penalized_max_corr_test_stat_selected, L_threshold] = ...
                 Escanciano_Lobato_maxlag_select_v7(y, MAX_L, q, filter_type)
% PURPOSE: perform the max-correlation version of Escanciano and Lobato's 
%             optimal lag selection.
%--------------------------------------------------------------------------
% USAGE: [maxlag_selected, penalized_max_corr_test_stat_selected, L_threshold] = ...
%                 Escanciano_Lobato_maxlag_select_v7(y, MAX_L, q, filter_type)
% where: y = data (n x 1)
%        MAX_L = the largest possible lag length (1 x 1)
%        q = tuning parameter (1 x 1)
%        filter_type = 0 if no filter is used.
%                      1 if mean filter is used.
%                      2 if AR(1) filter is used.
%                      3 if AR(2) filter is used.
%                      4 if GARCH(1,1) filter is used.
%--------------------------------------------------------------------------
% RETURNS: maxlag_selected = optimal lag length chosen from the candidates
%                                 {1, 2, ..., MAX_L}.    (1 x 1)
%          penalized_max_corr_test_stat_selected = max-correlation test
%                   statistic with L = maxlag_selected.    (1 x 1)
%          L_threshold = the smallest lag length among {1, 2, ..., MAX_L}
%                            that leads to the smaller penalty 
%                            (2 * maxlag)^(1/2)      (1 x 1)
%--------------------------------------------------------------------------
% See also: rho_hats_v4.m, penalized_max_corr_test_stat_v8.m
%--------------------------------------------------------------------------
% References: 
%    J. B. Hill and K. Motegi (2018). A Max-Correlation White Noise Test 
%        for Weakly Dependent Time Series. Working paper at the University
%        of North Carolina at Chapel Hill and Kobe University. 
%    J. C. Escanciano and I. N. Lobato (2009). An Automatic Portmanteau
%        Test for Serial Correlation. Journal of Econometrics, 151,
%        140-149.
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    September 24, 2018.
% -------------------------------------------------------------------------

n = size(y,1);          % sample size (1 x 1)
maxlags = (1:MAX_L)';   % a vecotr of L's considered (num_maxlags x 1)
num_maxlags = size(maxlags, 1);

% compute max-correlation test statistics with respect to actual data (as opposed to bootstrapped data)
max_corr_test_stats = zeros(num_maxlags,1);
for i = 1:num_maxlags 
     maxlag = maxlags(i);
     rho_hats = rho_hats_v4(y, maxlag, filter_type);
     max_corr_test_stat = sqrt(n) * max(abs(rho_hats));
     max_corr_test_stats(i) = max_corr_test_stat;
end    

% compute penelized max-correlation test statistics with respect to actual data
[penalized_max_corr_test_stats, L_threshold] = penalized_max_corr_test_stat_v8(max_corr_test_stats, maxlags, q, n);
[penalized_max_corr_test_stats_sorted, index] = sort(penalized_max_corr_test_stats, 'descend');
maxlag_selected = index(1);     % pick the lag that leads to the largest test statistic
penalized_max_corr_test_stat_selected = penalized_max_corr_test_stats_sorted(1);   % save the associated test statistic

end
