function Q = garch11_qml_crit_v2(theta, y)
% PURPOSE: Compute the NEGATIVE of quasi-likelihood function of GARCH(1,1).
%--------------------------------------------------------------------------
% USAGE: Q = garch11_qml_crit_v2(theta, y)
% where: theta = parameters (3 x 1)
%                  theta(1) = w
%                  theta(2) = a
%                  theta(3) = b
%         y = data (T x 1)
%--------------------------------------------------------------------------
% RETURNS: Q = the NEGATIVE of quasi-likelihood function of GARCH(1,1)
%--------------------------------------------------------------------------
% Specification: y(t) = sqrt(sigsq(t)) * e(t)
%                e(t) is error term
%                sigsq(t) = w + a * y(t-1)^2 + b * sigsq(t-1)
% -------------------------------------------------------------------------
% References: 
%   Jonathan B. Hill & Kaiji Motegi (2018). A Max-Correlation White Noise 
%       Test for Weakly Dependent Time Series. Working Paper at UNC
%       Chapel Hill and Kobe University. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    September 24, 2018.
% -------------------------------------------------------------------------

T = length(y);
w = theta(1);
a = theta(2);
b = theta(3);

Q = 0;
sigsq = w;
for t = 2:T
     sigsq = w + a * y(t-1)^2 + b * sigsq;
     Q = Q + log(sigsq) + (y(t)^2) / sigsq;
end

end