function [e_hat, dlogsigsq_hat, ddlogsigsq_hat] = garch_e_s_hat_v5(y, theta)
% PURPOSE: Compute standardized residual, d(log(sigsq)), dd(log(sigsq)) in GARCH(1,1) model.
%--------------------------------------------------------------------------
% USAGE: [e_hat, dlogsigsq_hat, ddlogsigsq_hat] = garch_e_s_hat_v5(y, theta)
% where: theta = parameters (3 x 1 or 1 x 3)
%                   theta(1) = w   constant
%                   theta(2) = a   ARCH parameter
%                   theta(3) = b   GARCH parameter
%-------------------------------------------------------------------------
% RETURNS: e_hat = standardized residual (T x 1)
%          dlogsigsq_hat = derivative of log(sigsq) wrt. w, a, and b   (T x 3)
%          ddlogsigsq_hat = derivative of dlogsigsq_hat wrt. w, a, and b  (3 x 3 x T)
%--------------------------------------------------------------------------
% Specification: y(t) = sqrt(sigsq(t)) * e(t)
%                e(t) is error term
%                sigsq(t) = w + a * y(t-1)^2 + b * sigsq(t-1)
% -------------------------------------------------------------------------
% References: 
%   Jonathan B. Hill & Kaiji Motegi (2018). A Max-Correlation White Noise 
%       Test for Weakly Dependent Time Series. Working Paper at UNC
%       Chapel Hill and Kobe University. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    September 24, 2018.
% -------------------------------------------------------------------------

T = length(y);        % sample size
w = theta(1);         % constant
a = theta(2);         % ARCH parameter
b = theta(3);         % GARCH parameter

% initial values 
sigsq = w * (1+b);    % sigma^2
dsigsq1 = 1 + b;      % d(sigsq) / dw
dsigsq2 = 0;          % d(sigsq) / da
dsigsq3 = w;          % d(sigsq) / db
ddsigsq11 = 0;        % dd(sigsq) / dwdw
ddsigsq12 = 0;        % dd(sigsq) / dwda
ddsigsq13 = 0;        % dd(sigsq) / dwdb
ddsigsq22 = 0;        % dd(sigsq) / dada
ddsigsq23 = 0;        % dd(sigsq) / dadb
ddsigsq33 = 0;        % dd(sigsq) / dbdb

% To minimize computational time, we create no vectors/matrices other than the three target objects.
e_hat = zeros(T,1);
dlogsigsq_hat = zeros(T,3);
ddlogsigsq_hat = zeros(3,3,T);
for t = 2:T
     ddsigsq11 = b * ddsigsq11;
     ddsigsq12 = b * ddsigsq12;
     ddsigsq13 = dsigsq1 + b * ddsigsq13;       % use dsigsq1 one period before
     ddsigsq22 = b * ddsigsq22;
     ddsigsq23 = dsigsq2 + b * ddsigsq23;       % use dsigsq2 one period before
     ddsigsq33 = 2 * dsigsq3 + b * ddsigsq33;   % use dsigsq3 one period before
     dsigsq1 = 1 + b * dsigsq1;
     dsigsq2 = y(t-1)^2 + b * dsigsq2;
     dsigsq3 = sigsq + b * dsigsq3;             % use sigsq one period before
     sigsq = w + a * y(t-1)^2 + b * sigsq;      % update sigsq
     
     e_hat(t) = y(t) / sqrt(sigsq);                          % save e_hat 
     dsigsq_vec = [dsigsq1, dsigsq2, dsigsq3];               % 1 x 3
     dlogsigsq_hat(t,:) = (1/sigsq) * dsigsq_vec;            % save s_hat
     ddsigsq_mat = [ddsigsq11, ddsigsq12, ddsigsq13;  
                    ddsigsq12, ddsigsq22, ddsigsq23;
                    ddsigsq13, ddsigsq23, ddsigsq33];
     ddlogsigsq_hat(:,:,t) = (1/sigsq) * (ddsigsq_mat - (1/sigsq) * (dsigsq_vec' * dsigsq_vec));    % save H_hat           
end

end