function [penalized_max_corr_test_stats, L_threshold] = penalized_max_corr_test_stat_v8(max_corr_test_stats, maxlags, q, n)
% PURPOSE: compute a vector of penalized max-correlation test statistics.
%--------------------------------------------------------------------------
% USAGE: [penalized_max_corr_test_stats, L_threshold] = penalized_max_corr_test_stat_v8(max_corr_test_stats, maxlags, q, n)
% where: max_corr_test_stats = max-correlation test statistics (num_maxlags x 1)
%        maxlags = a vector of maximum lags (num_maxlags x 1)
%                      max_corr_test_stats(i) is a max-corr statistic with L = maxlags(i).
%        q = tuning parameter (1 x 1)
%        n = sample size (1 x 1)
%--------------------------------------------------------------------------
% RETURNS: penalized_max_corr_test_stats = a vector of penalized max-correlation test statistics
%                                             (num_maxlags x 1)
%          L_threshold = the smallest lag length among the "num_maxlags" candidate lags
%                            that leads to the smaller penalty (2 * maxlag)^(1/2)      
%                              (1 x 1)
%--------------------------------------------------------------------------
% Note: max_corr_test_stats may be test statistics with respect to
%        correlations with first-order expansion or bootstrapped correlations.
% -------------------------------------------------------------------------
% References: 
%    J. B. Hill and K. Motegi (2018). A Max-Correlation White Noise Test 
%        for Weakly Dependent Time Series. Working paper at the University
%        of North Carolina at Chapel Hill and Kobe University. 
%    J. C. Escanciano and I. N. Lobato (2009). An Automatic Portmanteau
%        Test for Serial Correlation. Journal of Econometrics, 151,
%        140-149.
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    September 24, 2018.
% -------------------------------------------------------------------------

num_maxlags = size(maxlags, 1);     % the number of largest lags considered (1 x 1)
threshold = sqrt(q * log(n));       % 1 x 1

L_threshold = Inf;
penalties = zeros(num_maxlags,1);
for i = 1:num_maxlags 
     maxlag = maxlags(i);
     max_corr_test_stat = max_corr_test_stats(i);
     if max_corr_test_stat <= threshold  
         penalties(i) = (maxlag * log(n))^(1/2);    % larger penalty
     elseif max_corr_test_stat > threshold
         penalties(i) = (2 * maxlag)^(1/2);         % smaller penalty
         if maxlag < L_threshold
             L_threshold = maxlag;
         end    
     end    
end    

penalized_max_corr_test_stats = max_corr_test_stats - penalties;    % num_maxlags x 1

end
