function [pvalue, maxlags_boot_selected] = ...
           pvalue_DWB_maxcorr_expansion_optlag_v2(y, penalized_max_corr_test_stat_selected, filter_type, bsize, M, MAX_L, q)
% PURPOSE: compute dependent wild bootstapped p-value for penalized max-correlation test
%--------------------------------------------------------------------------
% USAGE: [pvalue, maxlags_boot_selected] = pvalue_DWB_maxcorr_expansion_optlag_v2
%                 (y, penalized_max_corr_test_stat_selected, filter_type, bsize, M, MAX_L, q)
% where: y = data (n x 1)
%        penalized_max_corr_test_stat_selected = 
%               penalized max-corr test statistic at optimal lag L* (1 x 1)
%        filter_type = 0 if no filter is used.
%                      1 if mean filter is used.
%                      2 if AR(1) filter is used.
%                      3 if AR(2) filter is used.
%                      4 if GARCH(1,1) filter is used.
%        bsize = block size (1 x 1)
%        M = # of bootstrap replications (scalar)
%        MAX_L = the largest possible lag length (1 x 1)
%        q = tuning parameter (1 x 1)
%--------------------------------------------------------------------------
% RETURNS: p_value = dependent wild bootstapped p-value (1 x 1)
%          maxlags_boot_selected = optimal lag length for each bootstrapped
%                                    sample (M x 1)
%--------------------------------------------------------------------------
% See also: rho_hats_DWB_expansion_v2.m, penalized_max_corr_test_stat_v8.m
%--------------------------------------------------------------------------
% References: 
%   Xiaofeng Shao (2011). A Bootstrap-Assisted Spectral Test of White 
%       Noise under Unknown Dependence. Journal of Econometrics, 162, 
%       213-224.
%   Jonathan B. Hill & Kaiji Motegi (2018). A Max-Correlation White Noise 
%       Test for Weakly Dependent Time Series. Working Paper at UNC
%       Chapel Hill and Kobe University. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    September 24, 2018.
% -------------------------------------------------------------------------

n = size(y,1);
maxlags = (1:MAX_L)';
num_maxlags = size(maxlags, 1);

counter = 0;
maxlags_boot_selected = zeros(M,1);
for b = 1:M    
    % compute bootstrapped max-correlation test statistics 
    max_corr_test_stats_boot = zeros(num_maxlags,1); 
    for i = 1:num_maxlags 
         maxlag = maxlags(i);
         rho_hats_boot = rho_hats_DWB_expansion_v2(y, maxlag, filter_type, bsize);
         max_corr_test_stat_boot = sqrt(n) * max(abs(rho_hats_boot));
         max_corr_test_stats_boot(i) = max_corr_test_stat_boot;
    end    
    
    % compute penelized max-correlation test statistics with respect to bootstrapped correlations
    penalized_max_corr_test_stats_boot = penalized_max_corr_test_stat_v8(max_corr_test_stats_boot, maxlags, q, n);
    [penalized_max_corr_test_stats_boot_sorted, index] = sort(penalized_max_corr_test_stats_boot, 'descend');
    maxlag_boot_selected = index(1);     % pick the lag that leads to the largest test statistic
    penalized_max_corr_test_stat_boot_selected = penalized_max_corr_test_stats_boot_sorted(1);   % save the associated test statistic
    
    % compared the penalized max-correlation test statistics from actual correlations and bootstrapped correlations
    counter = counter + (penalized_max_corr_test_stat_selected < penalized_max_corr_test_stat_boot_selected);
    maxlags_boot_selected(b) = maxlag_boot_selected;    % save the optimal lag length for b-th bootstrapped sample    
end 
pvalue = (1/M) * counter;    % bootstrapped p-value (1 x 1)
     
end
