function [rho_hats_boot, gamma_hats_boot] = rho_hats_DWB_expansion_v2(y, maxlag, filter_type, bsize)
% PURPOSE: compute bootstrapped autocorrelations at lags 1, ..., maxlag
%          by dependent wild bootstrap with the first order correlation expansion.
%--------------------------------------------------------------------------
% USAGE: [rho_hats_boot, gamma_hats_boot] = rho_hats_DWB_expansion_v2(y, maxlag, filter_type, bsize)
% where: y = data (N x 1)
%        maxlag = maximum lag length (1 x 1)
%        filter_type = 0 if no filter is used.
%                      1 if mean filter is used.
%                      2 if AR(1) filter is used.
%                      3 if AR(2) filter is used.
%                      4 if GARCH(1,1) filter is used.
%        bsize = block size of DWB (1 x 1)
%--------------------------------------------------------------------------
% RETURNS: rho_hats_boot = bootstrapped autocorrelations (maxlag_boot x 1)
%          gamma_hats_boot = bootstrapped autocovariances (maxlag_boot x 1)
%--------------------------------------------------------------------------
% Note: Various filters are allowed, and in that case the first order correlation 
%       expansion is executed.
%       DWB can require discarding some observations of y (i.e. n < N). 
%       If maxlag >= n, then maxlag_boot is set to be n - 1. 
%       Otherwise, maxlag_boot = maxlag.
% -------------------------------------------------------------------------
% See also: garch11_qml_crit_v2.m, garch_e_s_hat_v5.m
% -------------------------------------------------------------------------
% References: 
%   Xiaofeng Shao (2011). A Bootstrap-Assisted Spectral Test of White 
%       Noise under Unknown Dependence. Journal of Econometrics, 162, 
%       213-224.
%   Jonathan B. Hill & Kaiji Motegi (2018). A Max-Correlation White Noise 
%       Test for Weakly Dependent Time Series. Working Paper at UNC
%       Chapel Hill and Kobe University. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    October 7, 2018.
% -------------------------------------------------------------------------

N = size(y,1);                  % sample size
numblocks = floor(N / bsize);   % # of blocks
n = numblocks * bsize;          % effective sample size

if maxlag < n          % no adjustment of maxlag
    maxlag_boot = maxlag;
elseif maxlag >= n     % adjustment of maxlag
    maxlag_boot = n - 1;
end
y = y(1:n);     % cut initial values

% generate random numbers for DWB.
% Different blocks are independent from each other. Each block has same random numbers.                                            
z_boot_iid = randn(numblocks,1);             % numblocks x 1
z_boot = kron(z_boot_iid, ones(bsize,1));    % n x 1

% Different filters imply different dimensions and values of e_hat, D, A, and m.
if filter_type == 0        % no filter
    e_hat = y;                         % n x 1
    nobs = size(e_hat,1);
    D = zeros(1,1,maxlag_boot);        % 1 x 1 x maxlag_boot
    A = 0;                             % 1 x 1
    m = zeros(nobs,1);                 % n x 1
    z_boot = z_boot(1:n);              % n x 1
elseif filter_type == 1    % mean filter
    mu_hat = (1/n) * sum(y);
    e_hat = y - mu_hat;        % n x 1
    nobs = size(e_hat,1);
    D = zeros(1,1,maxlag_boot);        % 1 x 1 x maxlag_boot
    for h = 1:maxlag_boot
         y1 = y((h+1):nobs);
         y0 = y(1:(nobs-h));
         D(:,:,h) = (1/nobs) * sum((y1 - mu_hat) .* (y0 - mu_hat));
    end        
    A = 1;        % 1 x 1
    m = e_hat;    % n x 1
    z_boot = z_boot(1:n);         % n x 1    
elseif filter_type == 2    % AR(1) filter
    Y = y(2:n);
    X = y(1:(n-1));
    phi_hat = X \ Y;
    e_hat = Y - X * phi_hat;   % (n-1) x 1
    nobs = size(e_hat,1);
    D = zeros(1,1,maxlag_boot);        % 1 x 1 x maxlag_boot
    for h = 1:maxlag_boot
         D(:,:,h) = (1/nobs) * sum(e_hat(1:(nobs-h)) .* X((h+1):nobs)); 
    end    
    Ainv = (1/nobs) * (X' * X);        % 1 x 1
    A = Ainv \ eye(1);                 % 1 x 1    
    m = e_hat .* X;                    % nobs x 1
    z_boot = z_boot(2:n);              % nobs x 1    
elseif filter_type == 3    % AR(2) filter
    Y = y(3:n);
    X = [y(2:(n-1)), y(1:(n-2))];
    phi_hat = X \ Y;
    e_hat = Y - X * phi_hat;
    nobs = size(e_hat,1);
    D = zeros(2,1,maxlag_boot);        % 2 x 1 x maxlag_boot
    for h = 1:maxlag_boot
         D(:,:,h) = (1/nobs) * sum(repmat(e_hat(1:(nobs-h)), 1, 2) .* X((h+1):nobs, :))';     % 2 x 1 
    end    
    Ainv = (1/nobs) * (X' * X);       % 2 x 2
    A = Ainv \ eye(2);                % 2 x 2    
    m = repmat(e_hat, 1, 2) .* X;     % nobs x 2
%    z_boot = z_boot(2:n);             % nobs x 1        
    z_boot = z_boot(3:n);             % nobs x 1 
elseif filter_type == 4    % GARCH filter    
    % fmincon options
    optimization_options_fmincon = ...
         optimoptions('fmincon', 'Algorithm', 'interior-point', 'Display', 'off', ...
                      'TolX', 1e-5, 'TolFun', 1e-7, 'MaxFunEvals', 1000, ...
                      'MaxFunEvals', 20000, 'maxiter', 20000, 'GradObj', 'off');
    A_constraints = [-1,  0,  0;
                      0, -1,  0;
                      0,  0, -1;
                      0,  1,  1];
    b_constraints = [0, 0, 0, 1];
    theta_init = [1, 0.2, 0.5];
    theta_hat = fmincon(@garch11_qml_crit_v2, theta_init, A_constraints, b_constraints, ...
    [], [], [], [], [], optimization_options_fmincon, y);

    [e_hat, dlogsigsq_hat, ddlogsigsq_hat] = garch_e_s_hat_v5(y, theta_hat);

    nobs = size(e_hat,1); 
    s = 0.5 * dlogsigsq_hat;          % nobs x 3
    D = zeros(3,1,maxlag_boot);       % 3 x 1 x maxlag_boot
    for h = 1:maxlag_boot
         e_hat1 = e_hat((h+1):nobs);
         e_hat0 = e_hat(1:(nobs-h));
         s1 = s((h+1):nobs, :);
         s0 = s(1:(nobs-h), :);
         D(:,:,h) = (1/nobs) * sum(repmat(e_hat1 .* e_hat0, 1, 3) .* (s1 + s0))';    % 3 x 1
    end      
    
    Ainv_term1 = zeros(3,3);
    for t = 1:nobs
         st = s(t,:)';    % 3 x 1
         Ainv_term1 = Ainv_term1 + repmat(e_hat(t)^2, 3, 3) .* (st * st'); 
    end    
    Ainv_term1 = (2/nobs) * Ainv_term1;
    
    % Ess = (1/nobs) * (s' * s);    % 3 x 3
    % Ainv_term1 = 2 * Ess;
    
    ds = 0.5 * ddlogsigsq_hat;      % 3 x 3 x nobs
    Ainv_term2 = zeros(3,3);
    for t = 1:nobs
         Ainv_term2 = Ainv_term2 + repmat(e_hat(t)^2 - 1, 3, 3) .* ds(:,:,t); 
    end    
    Ainv_term2 = (1/nobs) * Ainv_term2;
    Ainv = Ainv_term1 - Ainv_term2;
    A = Ainv \ eye(3);                       % 3 x 3
    m = repmat(e_hat.^2 - 1, 1, 3) .* s;     % nobs x 3
end    

sigsq = (1/nobs) * sum(e_hat.^2);
gamma_hats_boot = zeros(maxlag_boot,1);
rho_hats_boot = zeros(maxlag_boot,1);
for h = 1:maxlag_boot
     e_hat1 = e_hat((h+1):nobs);                                   % (nobs - h) x 1
     e_hat0 = e_hat(1:(nobs-h));                                   % (nobs - h) x 1
     E_hat = e_hat1 .* e_hat0 - m((h+1):nobs,:) * A' * D(:,:,h);   % (nobs - h) x 1
     E_hat_mean = (1/nobs) * sum(E_hat);                           % 1 x 1
     z = z_boot((h+1):nobs);                                       % (nobs - h) x 1
     z_E_hat_mean = (1/nobs) * sum(z .* (E_hat - E_hat_mean));     % 1 x 1
     gamma_hats_boot(h) = z_E_hat_mean;
     rho_hats_boot(h) = (1/sigsq) * z_E_hat_mean;
end    
    
end
