function [rho_hats, gamma_hats] = rho_hats_v4(y, maxlag, filter_type)
% PURPOSE: compute sample autocorrelations at lags 1, ..., maxlag.
%--------------------------------------------------------------------------
% USAGE: [rho_hats, gamma_hats] = rho_hats_v4(y, maxlag, filter_type)
% where: y = data (n x 1)
%        maxlag = maximum lag length (1 x 1)
%        filter_type = 0 if no filter is used.
%                      1 if mean filter is used.
%                      2 if AR(1) filter is used.
%                      3 if AR(2) filter is used.
%                      4 if GARCH(1,1) filter is used.
%--------------------------------------------------------------------------
% RETURNS: rho_hats = sample autocorrelations (maxlag x 1)
%          gamma_hats = sample autocovariances (maxlag x 1)
%--------------------------------------------------------------------------
% Note: Filter is allowed, but the first order correlation expansion is not
%         allowed in this code. See rho_hats_expansion_v1.m for a version
%         with the first order correlation expansion.
% -------------------------------------------------------------------------
% See also: garch11_qml_crit_v2.m, garch_e_s_hat_v5.m
% -------------------------------------------------------------------------
% References: 
%   Jonathan B. Hill & Kaiji Motegi (2018). A Max-Correlation White Noise 
%       Test for Weakly Dependent Time Series. Working Paper at UNC
%       Chapel Hill and Kobe University. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    October 7, 2018.
% -------------------------------------------------------------------------

if filter_type == 0        % no filter
    e_hat = y;
elseif filter_type == 1    % mean filter
    n = size(y,1);
    mu_hat = (1/n) * sum(y);
    e_hat = y - mu_hat;
elseif filter_type == 2    % AR(1) filter
    n = size(y,1);
    Y = y(2:n);
    X = y(1:(n-1));
    phi_hat = X \ Y;
    e_hat = Y - X * phi_hat;
elseif filter_type == 3    % AR(2) filter
    n = size(y,1);
    Y = y(3:n);
    X = [y(2:(n-1)), y(1:(n-2))];
    phi_hat = X \ Y;
    e_hat = Y - X * phi_hat;
elseif filter_type == 4    % GARCH filter    
    % fmincon options
    optimization_options_fmincon = ...
         optimoptions('fmincon', 'Algorithm', 'interior-point', 'Display', 'off', ...
                      'TolX', 1e-5, 'TolFun', 1e-7, 'MaxFunEvals', 1000, ...
                      'MaxFunEvals', 20000, 'maxiter', 20000, 'GradObj', 'off');
    A_constraints = [-1,  0,  0;
                      0, -1,  0;
                      0,  0, -1;
                      0,  1,  1];
    b_constraints = [0, 0, 0, 1];
    theta_init = [1, 0.2, 0.5];
    theta_hat = fmincon(@garch11_qml_crit_v2, theta_init, A_constraints, b_constraints, ...
                           [], [], [], [], [], optimization_options_fmincon, y);
    [e_hat, ~, ~] = garch_e_s_hat_v5(y, theta_hat);
end    

nobs = size(e_hat,1);
sigsq_hat = (1/nobs) * sum(e_hat.^2);
gamma_hats = zeros(maxlag,1);
rho_hats = zeros(maxlag,1);
for h = 1:maxlag
     e_hat1 = e_hat((h+1):nobs);
     e_hat0 = e_hat(1:(nobs-h));
     gamma_hat = (1/nobs) * sum(e_hat1 .* e_hat0);
     gamma_hats(h) = gamma_hat;
     rho_hats(h) = gamma_hat / sigsq_hat;
end            
    
end
