%
% Jonathan B. Hill and Kaiji Motegi (2018). 
%   A Max-Correlation White Noise Test for Weakly Dependent Time Series.
%   Working paper at the UNC Chapel Hill and Kobe University.
%
% Matlab code on the max-correlation white noise test with Shao's dependent wild bootstrap (DWB).
%
% Subroutines directly called in this code:
%     1. garch11_gen_v2
%     2. ma_gen_v2
%     3. ar_gen_v2
%     4. bilinear_gen_v1
%     5. rho_hats_v4
%     6. rho_hats_WB_expansion_v1
%     7. rho_hats_DWB_expansion_v2
%
% Subroutines used in the subroutines above:
%     1. garch11_qml_crit_v2
%     2. garch_e_s_hat_v5
%
% References: 
%    X. Shao (2011). A Bootstrap-Assited Spectral Test of White Noise under
%        Unknown Dependence. Journal of Econometrics, 162, 213-224.
%
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%
% Last updated:
%    November 7, 2018.

%% Step 0: Initial Set-up
clear all
clc

% fix seed so that we always get the same result
defaultStream = RandStream.getGlobalStream;
defaultStream.reset(0);

tic;                       % start timing
start_date = now;          % start date
disp(['Job started: ', datestr(start_date)]);  % display start date

%% Step 1: Parameter Configuration
J = 1000;            % # of Monte Carlo samples
M = 500;             % # of bootstrap samples
etype = 1;           % error type 1 = IID
                     % error type 2 = GARCH
                     % error type 3 = MA
                     % error type 4 = AR
scenario = 7;        % scenario #1: IID process, mean filter, sample mean
                     % scenario #2: bilinear process, mean filter, sample mean
                     % scenario #3: AR(2) process, AR(2) filter, least squares
                     % scenario #4: AR(2) process, AR(1) filter, least squares
                     % scenario #5: GARCH(1,1) process, no filter
                     % scenario #6: GARCH(1,1) process, GARCH(1,1) filter, QML
                     % scenario #7: remote MA(6) process, mean filter, sample mean
                     % scenario #8: remote MA(12) process, mean filter, sample mean
                     % scenario #9: remote MA(24) process, mean filter, sample mean                     
n = 1000;            % sample size
boot_type = 2;       % 1 = wild bootstrap
                     % 2 = dependent wild bootstrap
delta = 1;           % Ln = max{5, [delta * n / log(n)]} with delta = 0.0, 0.5, 1.0.                   
                     
%% Step 2: Monte Carlo Simulations
maxlag = max(5, floor(delta * n / log(n)));    % largest lag length
nn = 2 * n;                                    % first n observations are burn-in      
if boot_type == 2
    bsize = floor(sqrt(n));       % block size of DWB
end    
   
if scenario == 1          % IID process, mean filter, sample mean
    nobs = n;
    filter_type = 1;
elseif scenario == 2      % bilinear process, mean filter, sample mean
    nobs = n;
    filter_type = 1;
elseif scenario == 3      % AR(2) process, AR(2) filter, least squares
    nobs = n - 2;        
    filter_type = 3;
elseif scenario == 4      % AR(2) process, AR(1) filter, least squares
    nobs = n - 1;        
    filter_type = 2;
elseif scenario == 5      % GARCH(1,1) process, no filter
    nobs = n;
    filter_type = 0;
elseif scenario == 6      % GARCH(1,1) process, GARCH(1,1) filter, QML
    nobs = n;
    filter_type = 4;
elseif scenario == 7      % remote MA(6) process, mean filter, sample mean
    nobs = n;
    filter_type = 1;
elseif scenario == 8      % remote MA(12) process, mean filter, sample mean
    nobs = n;       
    filter_type = 1;
elseif scenario == 9      % remote MA(24) process, mean filter, sample mean
    nobs = n;       
    filter_type = 1;
end 

rejections_001 = zeros(J,1);
rejections_005 = zeros(J,1);
rejections_010 = zeros(J,1);
parfor j = 1:J
%for j = 1:J
     % generate error term
     nu = randn(nn,1);
     if etype == 1            % IID
         e = nu;
     elseif etype == 2        % GARCH(1,1)
         param_error = [1, 0.2, 0.5];
         e = garch11_gen_v2(param_error, nu);
     elseif etype == 3        % MA(2)
         param_error = [0.5, 0.25];
         e = ma_gen_v2(param_error, nu);
     elseif etype == 4        % AR(1)
         param_error = 0.7;
         e = ar_gen_v2(param_error, nu);
     end    
     
     % generate y
     if scenario == 1          % IID process, mean filter, sample mean
         y = e;         
     elseif scenario == 2      % bilinear process, mean filter, sample mean
         param = 0.5;
         y = bilinear_gen_v1(param, e);
     elseif scenario == 3      % AR(2) process, AR(2) filter, least squares
         param = [0.3, -0.15];
         y = ar_gen_v2(param, e);
     elseif scenario == 4      % AR(2) process, AR(1) filter, least squares
         param = [0.3, -0.15];
         y = ar_gen_v2(param, e);
     elseif scenario == 5      % GARCH(1,1) process, no filter
         param = [1, 0.2, 0.5];
         y = garch11_gen_v2(param, e);
     elseif scenario == 6      % GARCH(1,1) process, GARCH(1,1) filter, QML
         param = [1, 0.2, 0.5];          
         y = garch11_gen_v2(param, e);
     elseif scenario == 7      % remote MA(6) process, mean filter, sample mean
         param_remote_MA = [zeros(1,5), 0.25];
         y = ma_gen_v2(param_remote_MA, e);  
     elseif scenario == 8      % remote MA(12) process, mean filter, sample mean
         param_remote_MA = [zeros(1,11), 0.25];
         y = ma_gen_v2(param_remote_MA, e);  
     elseif scenario == 9      % remote MA(24) process, mean filter, sample mean
         param_remote_MA = [zeros(1,23), 0.25];
         y = ma_gen_v2(param_remote_MA, e);        
     end           
     y = y((n+1):nn);
     
     rho_hats = rho_hats_v4(y, maxlag, filter_type);        % sample autocovariances (maxlag x 1)
     test_stat = sqrt(nobs) * max(abs(rho_hats));
     
     if boot_type == 1         % wild bootstrap
         counter = 0;
         for m = 1:M
              rho_hats_boot = rho_hats_WB_expansion_v1(y, maxlag, filter_type);              % bootstrapped autocorrelations (maxlag x 1)
              test_stat_boot = sqrt(nobs) * max(abs(rho_hats_boot));
              counter = counter + (test_stat_boot > test_stat);
         end
         pvalue = (1/M) * counter;       % bootstrapped p-value
     elseif boot_type == 2     % dependent wild bootstrap
         counter = 0;
         for m = 1:M
              rho_hats_boot = rho_hats_DWB_expansion_v2(y, maxlag, filter_type, bsize);      % bootstrapped autocorrelations (maxlag x 1)
              test_stat_boot = sqrt(nobs) * max(abs(rho_hats_boot));
              counter = counter + (test_stat_boot > test_stat);
         end
         pvalue = (1/M) * counter;       % bootstrapped p-value         
     end    
     
     rejections_001(j) = (pvalue < 0.01);         % rejections at 1% level 
     rejections_005(j) = (pvalue < 0.05);         % rejections at 5% level 
     rejections_010(j) = (pvalue < 0.10);         % rejections at 10% level 
end
     
rf_mat = (1/J) * [sum(rejections_001), sum(rejections_005), sum(rejections_010)];    % rejection frequencies
                                                                                            
%% Step 3: Close Computation
time = toc;      % finish timing
end_date = now;  % end date

% report computational time 
disp('*****************************************');
disp(['Job started: ', datestr(start_date)]);
disp(['Job finished: ', datestr(end_date)]);
disp(['Computational time: ', num2str(time), ' seconds.']);
disp(['Computational time: ', num2str(time / 60), ' minutes.']);
disp(['Computational time: ', num2str(time / 3600), ' hours.']);
disp('*****************************************');
disp(' ');

