%
% Jonathan B. Hill and Kaiji Motegi (2018). 
%   A Max-Correlation White Noise Test for Weakly Dependent Time Series.
%   Working paper at the UNC Chapel Hill and Kobe University.
%
% Matlab code on the max-correlation white noise test with Shao's dependent wild bootstrap (DWB) and automatic lag selection.
% In this code, we compute following quantities:
%     (1) rejection frequencies of the proposed test.
%     (2) empirical quantiles of bootstrapped p-values for size adjustment.
%
% Subroutines directly called in this code:
%     1. garch11_gen_v2
%     2. ma_gen_v2
%     3. ar_gen_v2
%     4. bilinear_gen_v1
%     5. Escanciano_Lobato_maxlag_select_v7
%     6. pvalue_DWB_maxcorr_expansion_optlag_v2
%
% Subroutines used in the subroutines above:
%     1. rho_hats_v4
%     2. penalized_max_corr_test_stat_v8
%     3. garch11_qml_crit_v2
%     4. garch_e_s_hat_v5
%     5. rho_hats_DWB_expansion_v2
%
% References: 
%    X. Shao (2011). A Bootstrap-Assited Spectral Test of White Noise under
%        Unknown Dependence. Journal of Econometrics, 162, 213-224.
%    J. C. Escanciano and I. N. Lobato (2009). An Automatic Portmanteau
%        Test for Serial Correlation. Journal of Econometrics, 151, 140-149. 
%
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%
% Last updated:
%    November 7, 2018.
%

%% Step 0: Initial Set-up
clear all
clc

% fix seed so that we always get the same result
defaultStream = RandStream.getGlobalStream;
defaultStream.reset(0);

tic;                       % start timing
start_date = now;          % start date
disp(['Job started: ', datestr(start_date)]);  % display start date

%% Step 1: Parameter Configuration
J = 1000;            % # of Monte Carlo samples
M = 500;             % # of bootstrap samples
etype = 1;           % error type 1 = IID
                     % error type 2 = GARCH
                     % error type 3 = MA
                     % error type 4 = AR
scenario = 7;        % scenario #1: IID process, mean filter, sample mean
                     % scenario #2: bilinear process, mean filter, sample mean
                     % scenario #3: AR(2) process, AR(2) filter, least squares
                     % scenario #4: AR(2) process, AR(1) filter, least squares
                     % scenario #5: GARCH(1,1) process, no filter
                     % scenario #6: GARCH(1,1) process, GARCH(1,1) filter, QML
                     % scenario #7: remote MA(6) process, mean filter, sample mean
                     % scenario #8: remote MA(12) process, mean filter, sample mean
                     % scenario #9: remote MA(24) process, mean filter, sample mean                     
n = 1000;            % sample size
q = 3.25;            % tuning parameter on automatic lag selection 
                     % q = 1.50, 1.75, ..., 4.25, 4.50.
exponent = 4/3;      % tuning parameters on largest possible lag length
delta = 1.5;         % MAX_L must be o(n / log(n)), hence a natural specification is MAX_L = floor(delta * n / (log(n))^exponent).
                     % If exponent = 3/2 and delta = 1, then MAX_L = {10, 19, 32, 55} for n = {100, 250, 500, 1000}.
                     % If exponent = 4/3 and delta = 1.5, then MAX_L = {19, 38, 65, 114} for n = {100, 250, 500, 1000}.

%% Step 2: Monte Carlo Simulations
MAX_L = floor(delta * n / (log(n))^exponent);      % largest possible lag length    
nn = 2 * n;                                        % first n observations are burn-in      
bsize = floor(sqrt(n));                            % block size of the dependent wild bootstrap

rejections_001 = zeros(J,1);
rejections_005 = zeros(J,1);
rejections_010 = zeros(J,1);
maxlag_selected_vec = zeros(J,1);
pvalue_vec_optimal = zeros(J,1);
parfor j = 1:J
%for j = 1:J
     % generate error term
     nu = randn(nn,1);
     if etype == 1            % IID
         e = nu;
     elseif etype == 2        % GARCH(1,1)
         param_error = [1, 0.2, 0.5];
         e = garch11_gen_v2(param_error, nu);
     elseif etype == 3        % MA(2)
         param_error = [0.5, 0.25];
         e = ma_gen_v2(param_error, nu);
     elseif etype == 4        % AR(1)
         param_error = 0.7;
         e = ar_gen_v2(param_error, nu);
     end    
     
     % generate y and specify filter
     if scenario == 1          % IID process, mean filter, sample mean
         y = e;
         y = y((n+1):nn);
         filter_type = 1;
     elseif scenario == 2      % bilinear process, mean filter, sample mean
         param = 0.5;
         y = bilinear_gen_v1(param, e);
         y = y((n+1):nn);
         filter_type = 1;
     elseif scenario == 3      % AR(2) process, AR(2) filter, least squares
         param = [0.3, -0.15];
         y = ar_gen_v2(param, e);
         y = y((n+1):nn);
         filter_type = 3;
     elseif scenario == 4      % AR(2) process, AR(1) filter, least squares
         param = [0.3, -0.15];
         y = ar_gen_v2(param, e);
         y = y((n+1):nn);
         filter_type = 2; 
     elseif scenario == 5      % GARCH(1,1) process, no filter
         param = [1, 0.2, 0.5];
         y = garch11_gen_v2(param, e);
         y = y((n+1):nn);
         filter_type = 0;
     elseif scenario == 6      % GARCH(1,1) process, GARCH(1,1) filter, QML
         param = [1, 0.2, 0.5];          
         y = garch11_gen_v2(param, e);
         y = y((n+1):nn);
         filter_type = 4;
     elseif scenario == 7      % remote MA(6) process, mean filter, sample mean
         param_remote_MA = [zeros(1,5), 0.25];
         y = ma_gen_v2(param_remote_MA, e);  
         y = y((n+1):nn);
         filter_type = 1;
     elseif scenario == 8      % remote MA(12) process, mean filter, sample mean
         param_remote_MA = [zeros(1,11), 0.25];
         y = ma_gen_v2(param_remote_MA, e);  
         y = y((n+1):nn);
         filter_type = 1;
     elseif scenario == 9      % remote MA(24) process, mean filter, sample mean
         param_remote_MA = [zeros(1,23), 0.25];
         y = ma_gen_v2(param_remote_MA, e);  
         y = y((n+1):nn);
         filter_type = 1;                  
     end                 
          
     % compute automatic lag for the actual max-correlation test statistic
     [maxlag_selected, penalized_max_corr_test_stat_selected] = Escanciano_Lobato_maxlag_select_v7(y, MAX_L, q, filter_type);
     maxlag_selected_vec(j) = maxlag_selected;         % record optimal lag
          
     % perform DWB, where automatic lag is computed in each bootstrap sample
     pvalue = pvalue_DWB_maxcorr_expansion_optlag_v2(y, penalized_max_corr_test_stat_selected, filter_type, bsize, M, MAX_L, q);
     pvalue_vec_optimal(j) = pvalue;              % record bootstrapped p-values
     rejections_001(j) = (pvalue < 0.01);         % rejections at 1% level 
     rejections_005(j) = (pvalue < 0.05);         % rejections at 5% level 
     rejections_010(j) = (pvalue < 0.10);         % rejections at 10% level 
end
     
rf_mat = (1/J) * [sum(rejections_001), sum(rejections_005), sum(rejections_010)];    % rejection frequencies
quantiles_pvalues = quantile(pvalue_vec_optimal, [0.01, 0.05, 0.10]);                % quantiles of p-values 
                                                                                     % these quantiles can be used for size adjustment in Figure 1. 
median_auto_lag = median(maxlag_selected_vec);          % median of automatic lags of actual test statistics
                                                        % This is reported in Table 2, but DWB is redundant in that regard.
                                     
%% Step 3: Close Computation
time = toc;      % finish timing
end_date = now;  % end date

% report computational time 
disp('*****************************************');
disp(['Job started: ', datestr(start_date)]);
disp(['Job finished: ', datestr(end_date)]);
disp(['Computational time: ', num2str(time), ' seconds.']);
disp(['Computational time: ', num2str(time / 60), ' minutes.']);
disp(['Computational time: ', num2str(time / 3600), ' hours.']);
disp('*****************************************');
disp(' ');

